import logging

from samanage import Samanage

logger = logging.getLogger(__name__)

class IncidentRecord:
    """
        TODO:  
    """
    def __init__(self, parent=None, **kwargs):
        self.number = kwargs.get('number', None)

        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return f"<Incident: {self.number}>" 

    
class Incident:
    _endpoints = {
        'incidents': '/incidents.json',
        'get_incident': '/incidents/{id}.json',
        'search': '/search.json?q={keyword}',
    }

    _required_fields = (
        'name',
        'requester',
        'priority',
    )

    incident_constructor = IncidentRecord

    def __init__(self, parent=None, *args, **kwargs):

        if parent is None:
            raise ValueError('Parent not specified.')
        self.con = parent.con
        self.base_url = parent.base_url

    def get(self, id, params=None, layout=None):
        """
            Return a single incident or page
        """
        path = 'https://api.samanage.com' + self._endpoints.get('get_incident').format(id=id)
        if layout:
            path = path + '?layout=long'
        response = self.con.get(path, params=params if params else None)

        if not response:
            return None
        data = response.json()
        return self.incident_constructor(**data) 

    def get_all(self, url=None, params=None, layout=None):
        """
            Return all incidents based on the url provided.
        """
        collections=None

        if url is None:
            path = 'https://api.samanage.com' + self._endpoints.get('incidents')
        else:
            path = url

        response = self.con.get(path, params=params if params else None)
        collections=response.json()
        while 'next' in response.links:
            response = self.con.get(response.links['next']['url'], params=params if params else None) 
            collections.extend(response.json())
        if not response:
            return None
        return [self.incident_constructor(**incident) for incident in collections]

    def search(self, keyword, **params):
        path = self.base_url + self._endpoints.get('search').format(keyword=keyword)
        self.con.get(path)

    def create(self, data):
        path = self.base_url + self._endpoints.get('incidents')

        if isinstance(data, dict):
            if 'incident' in data:
                if self._required_fields not in data.get('incident'):
                    raise ValueError('Fields: ' + ', '.join(self._required_fields) + ' is required.')
        else:
            raise TypeError('Payload must be a dict.')
        return self.con.create(path, data=data)

    def update(self, id, payload):
        path = self.base_url + self._endpoints.get('get_incident').format(id=id)

        if not isinstance(payload, dict):
            raise ValueError('Payload must be a dict.')
        return self.con.put(path, json=payload)

    def delete(self, id):
        path = self.base_url + self._endpoints.get('get_incident').format(id=id)
        return self.con.delete(path)

    def filter(self):
        pass



import logging
from .connection import Connection

logger = logging.getLogger(__name__)

SAMANAGE_API_VERSION = 'v2.1'
SAMANAGE_BASE_URL = 'https://api.samanage.com'

class Samanage:

    connection_constructor = Connection

    def __init__(self, token, base_url=None, **kwargs):

        headers = kwargs.get('headers', None)
        if token:
            if headers is None:
                kwargs['headers'] = {
                    'Accept': f'application/vnd.samanage.{SAMANAGE_API_VERSION}+json',
                    'Content-Type': 'application/json',
                    'X-Samanage-Authorization': f'Bearer {token}',
                }
            self.con = self.connection_constructor(token, **kwargs)
        else:
            raise ValueError("Token must be provided.") 

        if base_url is None:
            self.base_url = SAMANAGE_BASE_URL 
        else:
            self.base_url = base_url

    def incident(self, **kwargs):
        from .incidents import Incident
        return Incident(parent=self)

    def comment(self, **kwargs):
        from .comments import Comment 
        return Comment(parent=self)

    def change(self, **kwargs):
        pass

    def user(self, **kwargs):
        from .users import User
        return User(parent=self)

    def site(self, **kwargs):
        from .sites import Site
        return Site(parent=self) 


import logging
from samanage import incidents

from samanage import Samanage
from samanage.incidents import IncidentRecord

logger = logging.getLogger(__name__)

class UserRecord(IncidentRecord):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)

    def __repr__(self):
        super().__repr__()
        return f"<User: {self.id}>"

class User:
    _endpoints = {
        'users': '/users.json',
        'user': '/users/{id}.json'
    }

    _required_fields = (
        'email',
    )

    user_constructor = UserRecord

    def __init__(self, parent=None, *args, **kwargs):
        if parent is None:
            raise ValueError('Parent is not specified.')
        self.con = parent.con
        self.base_url = parent.base_url

    def get(self, id, params=None):
        path = self.base_url + self._endpoints.get('user').format(id=id)
        response = self.con.get(path, params=params)

        if not response:
            return None
        data = response.json()
        return self.user_constructor(**data)

    def get_all(self, params=None):
        collections = None

        path = self.base_url + self._endpoints.get('users')
        response = self.con.get(path, params=params)
        collections = response.json()

        while 'next' in response.links:
            response = self.con.get(response.links['next']['url'], params=params if params else None) 
            collections.extend(response.json())

        if not response:
            return None
        return collections

    def update(self, id, payload):
        path = self.base_url + self._endpoints.get('user').format(id=id)

        if not isinstance(payload, dict):
            raise ValueError('Payload must be a dict.')
        return self.con.put(path, json=payload)

    def delete(self, id):
        path = self.base_url + self._endpoints.get('user').format(id=id)
        return self.con.delete(path)




import logging

from enum import Enum

logger = logging.getLogger(__name__)

class Resource(Enum):
    CATALOG = 'incidents' 
    CHANGE = 'changes' 
    COMMENT = 'incidents' 
    INCIDENT = 'incidents' 
    PROBLEM = 'problems' 
    SITE= 'sites' 
    USER = 'users' 

class BaseApi:
    """ Base class for all interaction with Samanage API """

    def __init__(self, resource=None, **kwargs):
        # self._base_url = build_base_url() 

    def build_url(self, endpoint):
        if endpoint.startswith('/'):
            endpoint = endpoint[1:]

        return f"{self._base_url}/{endpoint}"

    def build_url(self):
        pass


__title__ = 'samanage-py'
__description__ = 'A package to easily connect to Samanage API' 
__url__ = 'https://github.com/rodneymandap/samanage-py'
__version__ = '0.1.8'
__author__ = 'Rodney Jan Mandap'
__author_email__ = 'rodneyjan.mandap@gmail.com'
__copyright__ = 'Copyright (c) 2021 Rodney Jan Mandap'



from samanage import Samanage

class Site(Samanage):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resource_url = self.options.get('site') 

    def create_site(self, payload):
        if not isinstance(payload, dict):
            raise ValueError("Payload must be a dict.")

        path = f'{self.base_url}/{self.resource_url}'
        return self.create(path, payload)

    def get_site(self, id, layout=None):
        path = f'{self.base_url}/sites/{id}.json'
        if layout:
            path = path + '?layout=long'
        return self.get(path)

    def get_all_sites(self, layout=None):
        path = f"{self.base_url}/{self.resource_url}"
        if layout:
            path = path + '?layout=long'
        return super().get_all(path)

    def update_site(self, payload):
        path = f"/{self.resource_url}"
        return self.update(path, payload)

    def delete_site(self, id):
        path = f'/sites/{id}.json'
        return self.delete(path)


import os
import setuptools

dito = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(dito, 'samanage', '__version__.py'), 'r', encoding='utf-8') as f:
    exec(f.read(), about)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name=about['__title__'],
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    description=about['__description__'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=about['__url__'],
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['requests'],
)